/**
 * Плагин для управления токена Laravel Passport.
 */
const VuePassportToken = {
    /**
     * Инъекция плагина.
     * 
     * @param {*} Vue Vue.js
     * @param {*} options Дополнительные параметры.
     */
    install(Vue, options) {
        Vue.mixin({
            mounted() {
                if (!window.wasLoaded) {
                    window.wasLoaded = true;
                    console.log('Laravel passport-tokens plugin was loaded!');
                }
            }
        });
        /**
         * Расширяем Vue методом $auth для доступа к функционалу управления токенами.
         *
         * @type {{setToken(string, int, string=): boolean, isAuthenticated: function(): (boolean|*), getToken: function(): (string | null), tokenWasExpired(): *, removeTokens(): void}}
         */
        Vue.prototype.$auth = {

            /**
             * Установка токенов.
             * 
             * @param {string} access_token 
             * @param {int} expires_in 
             * @param {string} refresh_token 
             */
            setToken: (access_token, expires_in, refresh_token = undefined) => {

                if (access_token === undefined || access_token.length < 1) {
                    console.warn(`Внимание! Не удалось установить токен: ${access_token}, т.к. он не задан или пуст!`);
                    return false;
                }

                localStorage.setItem('access_token', access_token);

                if (expires_in !== undefined) {
                    localStorage.setItem('expires_in', expires_in + new Date());
                }

                if (refresh_token !== undefined) {
                    localStorage.setItem('refresh_token', refresh_token);    
                }

                return true;
            },

            /**
             * Проверить авторизованность (наличие установленного токена и время его жизни).
             */
            isAuthenticated: () => !this.tokenWasExpired() && this.getToken(),

            /**
             * Получить токен доступа.
             */
            getToken: () => localStorage.getItem('access_token'),

            /**
             * Проверить, истекло ли время жизни токена.
             */
            tokenWasExpired: () => {
                const expires_in = localStorage.getItem('expires_in');

                return !expires_in ? false : new Date() >= expires_in;
            },

            /**
             * Удалить токены из хранилища.
             */
            removeTokens() {
                localStorage.removeItem('access_token');
                localStorage.removeItem('expires_in');
                localStorage.removeItem('refresh_token');
            }
        }
    }
};

export default VuePassportToken;
